import os

default_client_config = {}


def configure(newsorg_api_key: str | None = None) -> None:
    global default_client_config

    if newsorg_api_key is None:
        newsorg_api_key = os.getenv("NEWSORG_API_KEY")

    default_client_config = {
        "newsorg_api_key": newsorg_api_key,
    }



import string
from datetime import datetime


def to_datetime(date_string: str) -> datetime:
    try:
        # Replace is added to support both 3.10 and 3.11
        # See: https://docs.python.org/3/library/datetime.html#datetime.datetime.fromisoformat
        transformed = datetime.fromisoformat(date_string.replace("Z", "+00:00"))
    except ValueError:
        try:
            transformed = datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%SZ")
        except ValueError:
            try:
                transformed = datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
            except ValueError:
                transformed = datetime.strptime(date_string, "%a, %d %b %Y %H:%M:%S %z")

    return transformed


def slugify(sentence: str) -> str:
    return (
        sentence.lower()
        .translate(str.maketrans("", "", string.punctuation))
        .replace(" ", "-")
    )



from dataclasses import dataclass, field
from enum import Enum

from newspy.shared import utils
from newspy.shared.models import (
    Publication,
    Publisher,
    Source,
    Country,
    Language,
)


class NewsorgCategory(str, Enum):
    BUSINESS = "business"
    ENTERTAINMENT = "entertainment"
    GENERAL = "general"
    HEALTH = "health"
    SCIENCE = "science"
    SPORTS = "sports"
    TECHNOLOGY = "technology"


@dataclass
class NewsorgArticlesReq:
    """Newsorg API article requests"""

    url: str
    params: dict | None = None


@dataclass
class NewsorgSource:
    name: str
    id: str | None = field(default=None)
    description: str | None = field(default=None)
    url: str | None = field(default=None)
    category: NewsorgCategory | None = field(default=None)
    language: Language | None = field(default=None)
    country: Country | None = field(default=None)

    def to_publisher(self) -> Publisher:
        return Publisher(id=self.id, name=self.name, source=Source.NEWSORG)


@dataclass
class NewsorgSourceRes:
    """Newsorg API article source response"""

    status: str
    sources: list[NewsorgSource]

    def __post_init__(self):
        if isinstance(self.sources, list):
            self.sources = [
                NewsorgSource(**source) if isinstance(source, dict) else source
                for source in self.sources
            ]


@dataclass
class NewsorgArticle:
    """Newsorg API articles"""

    source: NewsorgSource
    author: str | None
    title: str
    description: str
    url: str
    urlToImage: str
    publishedAt: str
    content: str

    def __post_init__(self):
        if isinstance(self.source, dict):
            self.source = NewsorgSource(**self.source)

    def to_publication(self) -> Publication:
        publisher = Publisher(
            id=self.source.id,
            name=self.source.name,
            source=Source.NEWSORG,
        )

        return Publication(
            slug=f"{utils.slugify(self.source.name)}-{utils.slugify(self.title)}",
            url=self.url,
            url_to_image=self.urlToImage,
            title=self.title,
            abstract=self.description,
            author=self.author,
            publisher=publisher,
            published=utils.to_datetime(self.publishedAt),
        )


@dataclass
class NewsorgArticlesRes:
    """Response from the Newsorg API"""

    status: str
    totalResults: int
    articles: list[NewsorgArticle]

    def __post_init__(self):
        if isinstance(self.articles, list):
            self.articles = [
                NewsorgArticle(**article) if isinstance(article, dict) else article
                for article in self.articles
            ]



import logging
from dataclasses import dataclass
from enum import Enum

from newspy.shared import utils
from newspy.shared.models import Publication, Publisher, Source, Language

logger = logging.getLogger(__name__)


@dataclass
class RssArticleMediaContent:
    height: str
    width: str
    url: str
    medium: str | None


@dataclass
class RssArticleContent:
    base: str
    value: str


class RssCategory(str, Enum):
    BUSINESS = "business"
    FINANCIAL = "financial"
    ENTERTAINMENT = "entertainment"
    GENERAL = "general"
    HEALTH = "health"
    SCIENCE = "science"
    SPORTS = "sports"
    TECHNOLOGY = "technology"


@dataclass
class RssSource:
    id: str
    name: str
    description: str
    url: str
    category: RssCategory
    language: Language

    def to_publisher(self) -> Publisher:
        return Publisher(id=self.id, name=self.name, source=Source.RSS)


@dataclass
class RssArticle:
    title: str
    description: str
    url: str
    published: str

    def to_publication(self, publisher: Publisher) -> Publication:
        try:
            return Publication(
                slug=f"{utils.slugify(publisher.name)}-{utils.slugify(self.title)}",
                url=self.url,
                url_to_image=None,
                title=self.title,
                abstract=self.description,
                author=None,
                publisher=publisher,
                published=utils.to_datetime(self.published),
            )
        except ValueError:
            logger.warning(
                msg=f"Failed to parse the article with title: {self.title}, publisher: {publisher.name}"
            )
            pass


@dataclass
class RssArticleRes:
    sources: list[RssSource]

    def __post_init__(self):
        if isinstance(self.sources, list):
            self.sources = [
                RssSource(**source) if isinstance(source, dict) else source
                for source in self.sources
            ]



import logging
from enum import Enum

from newspy import client
from newspy.newsorg.models import (
    NewsorgArticlesRes,
    NewsorgCategory,
    NewsorgSourceRes,
    NewsorgSource,
    NewsorgArticle,
)
from newspy.shared.exceptions import NewspyException
from newspy.shared.http_client import HttpClient, HttpMethod
from newspy.shared.models import Language, Country

logger = logging.getLogger(__name__)

BASE_URL = "https://newsapi.org/v2"


class NewsorgEndpoint(str, Enum):
    EVERYTHING = "EVERYTHING"
    TOP_HEADLINES = "TOP_HEADLINES"
    SOURCES = "SOURCES"


def create_url(endpoint: NewsorgEndpoint) -> str:
    match endpoint:
        case NewsorgEndpoint.EVERYTHING:
            return f"{BASE_URL}/everything"
        case NewsorgEndpoint.TOP_HEADLINES:
            return f"{BASE_URL}/top-headlines"
        case NewsorgEndpoint.SOURCES:
            return f"{BASE_URL}/top-headlines/sources"
        case _:
            raise NewspyException(
                msg=f"The endpoint has to be one of the following values: 'EVERYTHING' or 'TOP_HEADLINES' or 'SOURCES'",
            )


def _create_params(page_size: int = None, page: int = None) -> dict[str, str]:
    newsorg_api_key = client.default_client_config.get("newsorg_api_key")

    if newsorg_api_key is None:
        raise NewspyException(
            msg="The Newsorg API key is not configured. Please configure it by calling the configure function.",
        )

    if page_size is None and page is None:
        return {"apiKey": newsorg_api_key}

    return {"apiKey": newsorg_api_key, "pageSize": page_size, "page": page}


def get_articles(
    endpoint: NewsorgEndpoint = NewsorgEndpoint.TOP_HEADLINES,
    search_text: str | None = None,
    category: NewsorgCategory | None = None,
    country: Country | None = None,
    sources: list[NewsorgSource] | None = None,
) -> list[NewsorgArticle]:
    params = _create_params(page_size=100, page=1)

    if category and sources:
        raise NewspyException(
            msg="Choose either the category and sources attributes. Not both.",
        )

    if search_text:
        params["q"] = search_text
    if category and endpoint == NewsorgEndpoint.TOP_HEADLINES:
        params["category"] = category.name
    if country and endpoint == NewsorgEndpoint.TOP_HEADLINES:
        params["country"] = country.name
    if sources and len(sources) > 0:
        params["sources"] = ",".join([source.id for source in sources])

    http_client = HttpClient()
    resp_json = http_client.send(
        method=HttpMethod.GET,
        url=create_url(endpoint=endpoint),
        params=params,
    )

    try:
        article_res = NewsorgArticlesRes(**resp_json)
    except TypeError as exc:
        raise NewspyException(
            msg=f"Failed to validate the News Org articles response json: {resp_json}",
            reason=str(exc),
        )

    return article_res.articles


def get_sources(
    category: NewsorgCategory | None = None,
    language: Language | None = None,
    country: Country | None = None,
) -> list[NewsorgSource]:
    params = _create_params()

    if category:
        params["category"] = category.value
    if language:
        params["language"] = language.value
    if country:
        params["country"] = country.value

    http_client = HttpClient()
    resp_json = http_client.send(
        method=HttpMethod.GET,
        url=create_url(endpoint=NewsorgEndpoint.SOURCES),
        params=params,
    )

    try:
        source_res = NewsorgSourceRes(**resp_json)
    except TypeError as exc:
        raise NewspyException(
            msg=f"Failed to validate the News Org sources response json: {resp_json}",
            reason=str(exc),
        )

    return source_res.sources
